/**
 * The MIT License (MIT)
 *
 * Copyright (C) 2013 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.apidesign.bck2brwsr.demo.serverside;

import java.io.Closeable;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import net.java.html.json.Context;
import org.apidesign.bck2brwsr.launcher.Launcher;
import org.glassfish.grizzly.http.server.HttpServer;
import org.glassfish.grizzly.http.server.ServerConfiguration;
import org.glassfish.jersey.grizzly2.httpserver.GrizzlyHttpContainer;
import org.glassfish.jersey.server.ContainerFactory;
import org.glassfish.jersey.server.ResourceConfig;

/** Server side of the chat application.
 */
@Path("/")
public final class ChatServerResource {
    public static void main(String... args) throws Exception {
        ResourceConfig rc = new ResourceConfig(ChatServerResource.class);
        GrizzlyHttpContainer c = ContainerFactory.createContainer(GrizzlyHttpContainer.class, rc);
        Closeable s = Launcher.showURL(
            "bck2brwsr", ChatClient.class.getClassLoader(),
            "/org/apidesign/bck2brwsr/demo/serverside/chat.html"
        );
        Launcher l = (Launcher) s;
        Callable<?> f = (Callable<?>) l;
        HttpServer server = (HttpServer) f.call();
        ServerConfiguration conf = server.getServerConfiguration();
        conf.addHttpHandler(c, "/chat");
        System.in.read();
        s.close();
    }
    
    private List<Message> msgs = new ArrayList<>();
    {
        Message welcome = new Message(Context.findDefault(Message.class));
        welcome.setUser("system");
        welcome.setComment("Welcome and enjoy!");
        welcome.setWhen(System.currentTimeMillis());
        msgs.add(welcome);
    }
    
    @Produces(MediaType.APPLICATION_JSON)
    @GET public Query getResources() {
        Query q = new Query(Context.findDefault(Query.class));
        q.getMessages().addAll(msgs);
        return q;
    }
    
    @Consumes(MediaType.APPLICATION_JSON)
    @PUT public void publish(Message msg) {
        if (msg != null) {
            msgs.add(msg);
        }
    }
}
